if __FILE__ == $0
  scrPath = File::dirname($0.gsub(/\\/, '/')).sub(/\/ldoce4$/, '')
  $LOAD_PATH.unshift(scrPath)     # XNvgpX[hpXɒǉ
end

require 'ldoce4/gaizi'
require 'ldoce4/cfg'

require "rexml/document"
include REXML

### Menu f[^
class DicMenu

  SEP_LINE = '' * 10  # ؂

  def initialize(outf)
    @f = outf
  end

  def close()
  end

  def do(source)

    doc = Document.new(source, { :ignore_whitespace_nodes => :all })
    root = doc.root

    @f.print '<p>', SEP_LINE, "</p>\n"

    root.elements.each do |e|
      case e.name
      when 'div'
        e.elements.each('span[@class="phrvbtitle"]') do |span|
          @f.print '<p><b>', span.text, "</b></p>\n"
        end
      when 'table'
        jump = e.attributes['id']
        e.elements.each('tr') do |tr|
          @f.print '<p>'
          tr.elements.each('td/span') do |span|
            cl = span.attributes['class']
            text = gaizi(span.text)
            case cl
            when 'menuNum'
              @f.print '<b>', "%2s" % text , '</b> '
            when 'mnu_signpost'
              @f.print '<a href="', jump, '"><b>&xLpost;', text, '&xRpost;</b></a>'
            when 'mnu_lexunit'
              @f.print '<a href="', jump, '"><b>', text, '</b></a>'
            when 'mnu_phrvbentry'
              @f.print '   <a href="', jump, '"><b>', text.strip, '</b></a>'
            else
              STDERR.print span.name, ':', cl, "\n"
            end
          end
          @f.print "</p>\n"
        end
      else
        STDERR.print e.name, "\n"
      end
    end

    @f.print '<p>', SEP_LINE, "</p>\n"

  end

end

### Usage note f[^
class UsageNote

  SEP_LINE = '' * 10  # ؂

  def initialize(outf)
    @f = outf
  end

  def close()
  end

  def do(source)

    doc = Document.new('<html>' + source + '</html>', { :ignore_whitespace_nodes => :all })
    root = doc.root

    @f.print '<p>', SEP_LINE, "</p>\n"

    root.elements.each do |e|
      cl = e.attributes['class']
      case cl
      when 'texte'
        block(e)
      else
        STDERR.print e.name, ':',  cl, "\n"
      end
    end

    @f.print '<p>', SEP_LINE, "</p>\n"

  end

  def block(node)

    cl = node.attributes['class']

    stag = etag = ''
    case cl
    when 'texte'              # {
      stag = '<p>'; etag = "</p>\n"
    else
      STDERR.print node.name, ':', cl, "\n"
      stag = '<p>'; etag = "</p>\n"
    end

    @f.print stag
    node.each do |n|
      case n.node_type
      when :text
        @f.print gaizi(n.value)  # if n.value !~ /^\n+\z/
      when :element
        case n.name
        when 'div', 'table', 'tr', 'td', 'p'
          block(n)
        else
          inline(n)
        end
      else
        STDERR.print n.node_type, "\n"
      end
    end
    @f.print etag

  end

  Ctags = {
#    'UEXA' => ['<i>', '</i>'],  # ᕶ    1072
    'UB'   => ['<b>', '</b>'],  # bold    6942
    'UR'   => ['', ''],         # roman   2631 (similar words, ...)
    'UI'   => ['<i>', '</i>'],  # italic   560
  }

  def inline(node)

    if node.name == 'br'
      # ubN̍Ō̗vfȂo͂Ȃ
      if node.next_element != nil
        @f.print "<br>\n"
      end
      return
    end

    cl = node.attributes['class']

    stag = ''; etag = ''
    case cl
    when 'UEXA'   # ᕶ
      prev = node.previous_element
      if prev != nil
        if prev.attributes['class'] == 'UEXA'
          stag = '| <i>'; etag = '</i>'  # 2 Ԗڈȍ~̗ᕶ
        else
          stag = ': <i>'; etag = '</i>'  # ŏ̗ᕶ
        end
      else
        stag = '<i>'; etag = '</i>'
      end
    when nil
     ;
    else
      if Ctags.key?(cl)
        stag = Ctags[cl][0]; etag = Ctags[cl][1]
      else
        if node.has_text?
          STDERR.puts cl
        end
      end
    end

    @f.print stag
    node.each do |n|
      case n.node_type
      when :text
        @f.print gaizi(n.value)   # if n.value !~ /^\n+\z/
      when :element
        inline(n)
      else
        STDERR.puts n.node_type
      end
    end
    @f.print etag

  end

end

### 摜f[^
class ImageData
  IMG_DIR = 'img'
  IMG_EXT = '.jpeg'

  def initialize(maindoc, outDir)
    @maindoc = maindoc
    @outDir = outDir.sub(/\/$/, '') + '/' + IMG_DIR
    Dir::mkdir(@outDir) unless File::exist?(@outDir)
  end

  def do(id)
    s = ''

    rec = @maindoc.getrec(id)
    idx = @maindoc.getbodyindex(rec)
    imgidx = mapidx = nil
    idx.each do |x|
      case x['packid']
      when PID_IMAGE     then imgidx = x  # ʏ摜
      when PID_IMAGE_MAP then imgidx = x  # }bv摜
      when PID_HTML_MAP  then mapidx = x  # }bv html
      end
    end
    if imgidx != nil
      img = @maindoc.getbody(imgidx)
      fname = id.to_s + IMG_EXT
      File::open(@outDir + '/' + fname, 'wb') do |f|
        f.write(img)
      end
      s = '<img src="' + IMG_DIR + '/' + fname + '">'
#      s = '<img src="' + IMG_DIR + '/' + fname + '" alt="' + title + '">'
      if mapidx != nil
        data = @maindoc.getbody(mapidx)
        s << htmlmap('<html>' + data + '</html>') 
      end
    end

    s
  end

  def htmlmap(source)
    doc = Document.new(source, { :ignore_whitespace_nodes => :all })
    root = doc.root

    s = ''
    root.elements.each do |elem|
      if elem.name == 'area'
        if elem.attributes['href'] =~ /OpenArea\('(\d+)'\)/
          id = $1
          word = elem.attributes['alt'].sub(/_.*$/, '')
          s << '<a href="' + id + '">' + word + '</a>, '
        end
      end
    end

    return s.sub(/, $/, '')
  end

end

### {
class DicBody

  EX_BULLET = '@E'      # ᕶp

  def initialize(outf, maindoc, outDir)
    @f = outf
    @maindoc = maindoc

    @menu = DicMenu::new(@f)
    @usage = UsageNote::new(@f)
    @image = ImageData::new(@maindoc, outDir)
  end

  def close()
  end

  def do(source)
     # 哮₷悤ɒuĂ
     source = source.gsub(%r!</span> <span class="hwdcomp">!n, ' ')

#    doc = Document.new('<html>' + source + '</html>', { :ignore_whitespace_nodes => :all })
    doc = Document.new('<html>' + source + '</html>')
    rootblock(doc.root)
  end

  def rootblock(root)

    menurec = usagerec = nil
    @woGroup = @imageid = nil
    @hasMenu = false

    root.elements.each do |e|
      result = ''
      case e.attributes['class']
      when "woGroup"              # ꌹ  14121
        @woGroup = e.deep_clone
        next
      when "pron"                 #   29206
        next
      when "unfolded"             # o  47636
        div_unfolded(e)

        rec = @maindoc.getrec(@id.to_i)
        idx = @maindoc.getbodyindex(rec)
        idx.each do |x|
          case x['packid']
          when PID_MENU        #  Menu
#STDERR.print 'Menu:', @id, "\n"
            menurec = x
            @hasMenu = true
          when PID_USAGE_NOTE  #  Usage note
#STDERR.print 'Usage Note:', @id, "\n"
            usagerec = x
          end
        end

        next
      when "infoHwd"              # oǉ  47636
        div_infoHwd(e)
        next
      when "sensehwd"             # `  86785

        if @imageid != nil
#STDERR.print 'Image:', @imageid, "\n"
          anchor = @image.do(@imageid.to_i)
          @f.print '<p>' + anchor + "</p>\n"
          @imageid = nil
        end
        if menurec != nil
          body = @maindoc.getbody(menurec)
          @menu.do(body)
          menurec = nil
        end

        if @woGroup != nil
          @f.print div_woGroup(@woGroup.root)
          @woGroup = nil
        end

        # div_sensehwd(e)
        result = block(e)
      when "Tail"                 # ǉ   8524
        # div_Tail(e)
        result = block(e)
      when "endOfEntry"           # ؂  47636
        if usagerec != nil
          body = @maindoc.getbody(usagerec)
          @usage.do(body)
        end
        @f.print "</dd>\n"
        next
      when nil
        # div_noclass(e)
        result = block(e)
      else
        STDERR.print e.name, ':', e.attributes['class'], "\n"
      end

      if result != ''
        result.gsub!(/<br>\n+(<\/?p>)/) { "\n" + $1 }
        result.gsub!(/<p><br>\n/, '<p>')
#        result.gsub!(/(<br>)+\n<p>/) { "\n<p>" }
#        result.gsub!(/<br>\n<\/p>\n<p>/) { "</p><p>\n" }
        result.gsub!(/<p>(<b>)? +/) { if $1 != nil then '<p><b>' else '<p>' end }
        result.gsub!(/-- see also /, '')
        result.gsub!(/-- synonym /, '')
        result.gsub!(/-- antonym /, '')
        result.gsub!(/-- opposite /, '')

        @f.print result
      end

    end

  end

  def div_woGroup(elem)
    result = ''; header = ''

    elem.each do |n|
      case n.node_type
      when :text
        result << gaizi(n.value) if n.value !~ /^\n+\z/
      when :element
        case n.name
        when 'table'
          result << '<p>['
          if header != ''
            result << ( header + '; ')
            header = ''
          end
          result << div_woGroup(n)
          result << "]</p>\n"
          result.gsub!(/ +; /, '; ')
          result.gsub!(/; +\]/, ']')
        when 'tr'
          result << div_woGroup(n)
          result << '; '
        when 'td'
          result << div_woGroup(n)
          result << ' '
        when 'br'
          ;
        else
          if n.attributes['class'] == 'sense'   # `敪(ꌹ) 805
            header = inline(n)
          else
            result << inline(n)
          end
       end
      else
        STDERR.puts n.node_type
      end
    end

    result
  end

  def div_unfolded(elem)

    @id = elem.attributes['id']
    @headword = @homograph = @wordclass = @longwordclass = ''
    @freq = ''

    elem.elements.each("table/tr/td[@class='hwdSelMM']/table/tr/td") do |e|
      case e.attributes['class']
      when 'hdSel'
        e.elements.each do |e2|
          case e2.attributes['class']
          when "headwordSelected"
            @headword = gaizi(e2.text) if e2.has_text?
          when "homographsSelected"
            @homograph = e2.text if e2.has_text?
          when "wordclassSelected"
            @longwordclass = e2.text if e2.has_text?
            @wordclass = shortenPOS(e2.text) if e2.has_text?
          else
          end
        end
      when 'freqS'
        @freq << 'S'
      when 'freqW'
        @freq << 'W'
      when 'freqNum1'
        @freq << '1'
      when 'freqNum2'
        @freq << '2'
      when 'freqNum3'
        @freq << '3'
      else
      end
    end

  end

  def div_infoHwd(elem)

    result = ''; hyphenhead = ''; title = ''; orthvar = ''

    elem.each do |n|
      case n.node_type
      when :text
        result << gaizi(n.value) if n.value !~ /^\n+\z/
      when :element
        case n.attributes['class']
        when 'HYPHENATION'
          hyphenhead = gaizi(n.text).strip
          if @homograph != ''
            title = @headword + '  ' + @homograph + ', ' + @longwordclass
          else
            title = @headword
          end
  #      when "ORTHVAR"               # \Lʌ`  1798
  #        orthvar = gaizi(n.text).strip
        when "PRONSLASHESR"          # IL 60824
          result << inline(n)
          if @wordclass != ''
            result << ' <i>' << @wordclass << '</i> '
            @wordclass = ''
          end
        when "GRAM"                  # @ 44431 ([countable], etc)
          if @wordclass != ''
            result << ' ' if result != ''
            result << ' <i>' << @wordclass << '</i> '
            @wordclass = ''
          end
          result << inline(n)
        when 'Inflections'
          result << block(n)
        else
          result << inline(n)
        end
      else
      end
    end

    title.gsub!(/<.+?>/, '') # ^CgɃ^O EBStudio  (H2O)

    @f.print '<dt id="', ID_CHAR, @id, '" title="', title, '">', hyphenhead, "</dt>\n"
    @f.print "<dd>\n"
    if @headword =~ /, (the|A)$/    # ̊폜oo^
      @f.print '<key type="\L">', $`, "</key>\n"
    end
    @f.print '<p>'
    s = ''
    s << '<sup>' << @homograph << '</sup>' if @homograph != ''
#    when "ORTHVAR"               # \Lʌ`  1798
#      if result =~ /^.+<\/dfn>/
#        s << $&
#        result = $'
#    end
    s << ' <b>' << @freq << '</b>' if @freq != ''
    s << ' <i>' << @wordclass << '</i>' if @wordclass != ''
    @f.print s + ' ' if s != ''
    @f.print result
    @f.print "</p>\n"

  end

  def div_Tail(elem)
  end

  def block(node)
    result = ''

    cl = node.attributes['class']

    stag = etag = ''
    case cl
    when "sensehwd"             #   86785
      stag = '<p>'; etag = "</p>\n"
    when "Tail"                 #    8524
      if node.size != 0
        stag = '<p>'; etag = "</p>\n"
      end
#    when "RunOn"
#      ;
    when nil
      if @hasMenu
        if node.name == 'table' || node.name == 'div'
          id = node.attributes['id']
          if id != nil
            stag = '<a name="' + id + '"></a>'; etag = ''
          end
        end
      end
    else
      case node.name
      when 'table', 'tr', 'td'
        ;
      when 'p'
        stag = '<p>'; etag = "</p>\n"
      else
        stag = '<p>'; etag = "</p>\n"
#        STDERR.puts cl
      end
    end

    result << stag
    node.each do |n|
      case n.node_type
      when :text
        result << gaizi(n.value) if n.value !~ /^\n+\z/
      when :element
        case n.name
        when 'div', 'table', 'tr', 'td', 'p'
          result << block(n)
        when 'xblkpron', 'xblspron', 'xbltoolhwd'
          ;
        else
          result << inline(n)
        end
      else
        p n.node_type
      end
    end
    result << etag

    result
  end

  def shortenPOS(s)
    s.gsub!(/noun/, 'n')
    s.gsub!(/phrasal verb/, 'phr v')
    s.gsub!(/verb/, 'v')
    s.gsub!(/adjective/, 'adj')
    s.gsub!(/adverb/, 'adv')
    s.gsub!(/conjunction/, 'conj')
    s.gsub!(/preposition/, 'prep')
    s.gsub!(/pronoun/, 'pron')
    s
  end

  def shortenGram(s)
    s.gsub!(/uncountable/, 'U')
    s.gsub!(/countable/, 'C')
    s.gsub!(/intransitive/, 'I')
    s.gsub!(/transitive/, 'T')
    s
  end

  def shortenGEO(s)
    s.gsub!(/British English/, 'BrE')
    s.gsub!(/American English/, 'AmE')
    s.gsub!(/Australian English/, 'AusE')
    s
  end

  def shortenPron(s)
    s.gsub!(/somebody/, 'sb')
    s.gsub!(/something/, 'sth')
    s
  end

  Ctags = {
  #  "headwordSelected"   => ['', ''],            # o 47636
  #  "homographsSelected" => ['<sup>', '</sup>'], # Ԍoԍ 47636
  #  "wordclassSelected"  => ['<i>', '</i>'],     # i 47636

  #  "REFHWD"                # Q 16602  id=
  #  "REFHWDB"               # Q  6057  bold? id=
    "REFHOMNUM"        => ['<sup>', '</sup>'],    # QƎԌԍ  1985

    "senseNb"          => ['<b>', '</b> '],  # `ԍ 86785
    "subsenseletter"   => ['<b>', '</b>'],   # ʌ`敪  3551  ( a), b),...)
    "sense"            => ['<b>', '</b>'],   # `敪(ꌹ) 805
    "example"          => ['<p>', "</p>\n"], # ᕶ 88310
    "GRAM"             => [' ', ' '],        # @ 44431 ([countable], etc)

    "REGISTERLAB"      => ['<i>', '</i>'],   # gp 16492
    "tran"             => ['<i>', '</i>'],   # O̖ 10689
    "lang"             => ['<i>', '</i>'],   # ꖼ  7605
    "GEO"              => ['<i>', '</i>'],   # n 11130
    "POS"              => ['<i>', '</i>'],   # i(o) 6940
    "wordclass"        => ['<i>', '</i>'],   # i  4003

    "deriv"            => ['<dfn>', '</dfn>'], # h  6768

#    "SYN"              => ['<b>', '</b>'],   # ދ`  5147
#    "OPP"              => ['<b>', '</b>'],   # `  1758
#    "RELATEDWD"        => ['<b>', '</b>'],   # ֘A  3746

#    "generation"       => ['', ' '],         #  Synonym, etc 15417

    "LINKWORD"         => ['<i>', '</i>'],   # Ȃ  3698 (also, etc)
    "cross_linkword"   => ['<i>', '</i>'],   # QƎȂ  2093 (at, etc)
    "CROSSREFTYPE"     => ['<i>', '</i>'],   # ݎQ  255 (see usage note, etc)
    "beforePTandPP"    => ['<i>', '</i>'],   #    888
    "beforePRESPART"   => ['<i>', '</i>'],   #    751
    "beforePASTPART"   => ['<i>', '</i>'],   #    114
    "beforePASTTENSE"  => ['<i>', '</i>'],   #    115
    "beforeT3PERSSING" => ['<i>', '</i>'],   #    187
    "beforeCOMP"       => ['<i>', '</i>'],   #    180
    "beforeSUPERL"     => ['<i>', '</i>'],   #    181
    "STRONG"           => ['<i>', '</i>'],   # `   167 (strong)

    "COLLO"            => ['<p><b>', "</b></p>\n"],   # A  7229
    "PROPFORM"         => ['<p><b>', "</b></p>\n"],   # ^ + ?  5912
    "PROPFORMPREP"     => ['<p><b>', "</b></p>\n"],   # + Ou  ? 7606
#    "phrvbhwdsel"      => ['<p><dfn>', ''],       # 哮  2288
#    "hwdcomp"          => ['', '</dfn></p>'],     # 哮̏  2288  (up, on, etc)

    "LEXUNIT"          => ['<b>', '</b>'],   # ? 17861
    "signpost"         => ['<b>', '</b>'],   # TC|Xg  8841
    "crossref"         => ['<b>', '</b>'],   # ݎQƌ(啶)  3743
    "lexvar"           => ['<b>', '</b>'],   # ʌ`?  2744

    "ORTHVAR"          => ['<dfn>', '</dfn>'],  # \Lʌ`  1798

    "FULLFORM"         => ['<b>', '</b>'],   # Zk`   887
    "ABBR"             => ['<b>', '</b>'],   # Zk`   352
    "AMEQUIV"          => ['<b>', '</b>'],   # Č`   759
    "BREQUIV"          => ['<b>', '</b>'],   # p`   720
    "PLURALFORM"       => ['<b>', '</b>'],   # `  1894
    "PTandPP"          => ['<b>', '</b>'],   # ߋ`/ߋ`  924
    "PRESPART"         => ['<b>', '</b>'],   # ݕ`   760
    "PASTPART"         => ['<b>', '</b>'],   # ߋ`   125
    "PASTTENSE"        => ['<b>', '</b>'],   # ߋ`   121
    "T3PERSSING"       => ['<b>', '</b>'],   # OP`   188
    "COMP"             => ['<b>', '</b>'],   # r   183
    "SUPERL"           => ['<b>', '</b>'],   # ŏ㋉   183
    "Unclassified"     => ['<b>', '</b>'],   # Ȍ`    11 (can't, etc)

    "PRON"             => ['', ''],          #  89779
    "PRONSLASHESG"     => ['', ''],          # JnL 60824
    "PRONSLASHESR"     => ['', ''],          # IL 60824
    "headwordSelected" => ['', ''],          # o 47636
    "HYPHENATION"      => ['', ''],          #  47628
    "definition"       => ['', ''],          # ` 88637
    "GLOSS"            => ['', ''],          #   7552 (=...)
    "REFSENSENUM"      => ['', ''],          # QƎ`ԍ  2214
    "refsense"         => ['', ''],          # QƎ`ԍ    87
    "SUFFIX"           => ['', ''],          # QƎsv     3
    "HWD"              => ['', ''],          #  ? 14121 (woGroup Aׂċvf)

  }

  def inline(node)
    result = ''

    if node.name == 'br'
      return "<br>\n"
    end

    cl = node.attributes['class']

    stag = ''; etag = ''
    case cl
    when "REFHWD"                # Q 16602  id=
      id = node.attributes['id']
      id = '0' if id == nil
      stag = '<a href="' + ID_CHAR + id + '">'; etag = '</a>'
    when "REFHWDB"               # Q  6057  bold?
      id = node.attributes['id']
      id = '0' if id == nil
      stag = ' <b><a href="' + ID_CHAR + id + '">'; etag = '</a></b>'
    when "puce"                  # ᕶ bullet 摜
      return EX_BULLET
    when "generation"            #  Synonym, etc 15417
      stag = ''; etag = ' '
      if node.previous_element == nil
        if node.parent.name == 'div'
          stag = "<br>\n"; etag = ' '
        end
      end
    when "SYN",                  # ދ`  5147
         "OPP",                  # `  1758
         "RELATEDWD"             # ֘A  3746
      if node.has_text?
        stag = '<b>'
        etag = '</b><key type="">' + gaizi(node.text) + '</key>'
      end
    when "phrvbhwdsel"           # 哮  2288
      # 㖼Ȃǂ폜`oɂ
      phr = gaizi(node.text).strip
      key = phr.gsub(/(something|somebody|)/, '').gsub(/ \/ /, ' ').gsub(/  +/, ' ').strip
      return '<p><dfn>' + key + '</dfn>' +
             if phr != key then ' [' + shortenPron(phr) + ']' else '' end +
             '</p>'
    when "preview noprint"       # 摜
      @imageid = node.attributes['id']
      return ''
    when nil
      case node.name
      when 'b'
        stag = '<b>'; etag = '</b>'
      when 'sup'
        stag = '<sup>'; etag = '</sup>'
      end
    else
      if Ctags.key?(cl)
        stag = Ctags[cl][0]; etag = Ctags[cl][1]
      else
        if node.has_text?
          STDERR.puts cl
        end
      end
    end

    node.each do |n|
      case n.node_type
      when :text
        result << gaizi(n.value) if n.value !~ /^\n+\z/
      when :element
        result << inline(n)
      else
        STDERR.puts n.node_type
      end
    end

    case cl
    when "GRAM"                  # @ 44431 ([countable], etc)
      result = shortenGram(result)
    when "LEXUNIT",              # ? 17861
         "COLLO",                # A  7229
         "PROPFORM",             # ^ + ?  5912
         "lexvar",               # ʌ`?  2744
         "phrvbhwdsel"           # 哮  2288
      result = shortenPron(result)
    when "GEO"                   # n 11130
      result = shortenGEO(result)
    when "POS",                  # i(o) 6940
         "wordclass"             # i  4003
      result = shortenPOS(result)
    end

    result = stag + result + etag if result != ''

    result
  end

end

if __FILE__ == $0

  RS = "\x01"

  maindoc = MainDoc::new('H:/ldoce4/ldoce-data/')
  doc = DicBody::new(STDOUT, maindoc, 'K:/')

  puts '<html>'
  puts '<head>'
  puts '<title>Longman Dictionary of Contemporary English - 4th Edition</title>'
  puts '</head>'
  puts '<body>'
  puts '<dl>'

  while rec = DATA.gets(RS)
    doc.do(rec.chomp(RS))
  end

  puts '</dl>'
  puts '</body>'
  puts '</html>'

=begin
  infname = ARGV.shift.gsub(/\\/, '/')
  RS = "\x01"

  menu = DicMenu::new(STDOUT)
  inf = open(infname)

  n = 0
  while rec = inf.gets(RS)
    break if n >= 10
    n += 1
    rec.chomp!(RS)
    menu.do(rec)
  end

  maindoc = MainDoc::new('H:/ldoce4/ldoce-data/')
  img = ImageData::new(maindoc, 'K:')
  [247064, 247065, 247083, 247100].each do |x|
    puts img.do(x)
  end

  usage = UsageNote::new(STDOUT)
  while data = DATA.gets("\x01")
    data.chomp!("\x01")
    usage.do(data)
  end
=end
end

__END__

