if __FILE__ == $0
  scrPath = File::dirname($0.gsub(/\\/, '/')).sub(/\/ldoce4$/, '')
  $LOAD_PATH.unshift(scrPath)     # XNvgpX[hpXɒǉ
end

require 'ldoce4/gaizi'
require 'ldoce4/cfg'

require "rexml/document"
include REXML

### ᕶ f[^
class Example

  EB_DIC_BULLET     = '- '  # Extra dictionary examples
  EB_BOOK_BULLET    = '= '  # Sentences from books, newspapers, etc.
  EB_UNKNOWN_BULLET = '? '  # error

  def initialize(outf, maindoc)
    @f = outf
    @maindoc = maindoc
  end

  def close()
  end

  def do(source)
    id, hwd, items = parse(source)
    printHTML(id, hwd, items) # if items.size > 0
  end

  def parse(source)
    id = hwd = nil
    items = []
    if source =~ /<div id="(\d+)" nb="\d+" hwd="(.+?)">/m
      id = $1; hwd = $2
      source.scan(%r!<p class="(\w+)"(?: pos="(\d+)")?>(.+?)</p>!m) do |cl, pos, text|
        case cl
        when "titledicphr"
          items.push('title' => text, 'ex' => [])
        when "corpex"
          if text =~ %r!/>(.+)\Z!
            items.last['ex'].push('text' => $1, 'pos' => pos)
          end
        end
      end
    end

    return id, hwd, items
  end

  def printHTML(id, hwd, items)

    # o
    if hwd =~ /^\n/   # Phrase Bank ᕶ
      hwd = hwd.sub(/\n/, '')
#      @f.print '<dt id="', ID_CHAR_EXA, id, '">', hwd, "</dt>\n"
      @f.print '<dt id="', ID_CHAR, id, '">', hwd, "</dt>\n"
      @f.print "<dd>\n"
      @f.print "<p></p>\n"
    else              # Dic ᕶ
      title = @maindoc.title(id.to_i)
      if title =~ /  (\d, )?/      # oAԌԍAiɕ
        hwd = $`; num = $1; parts = $'
      else
        hwd = title; num = nil; parts = ''
      end
      if num != nil
        @f.print '<dt id="', ID_CHAR_EXA, id, '" title="', title, '">', hwd, "</dt>\n"
        @f.print "<dd>\n"
        @f.print '<p>', num, '<i>', parts, "</i></p>\n"
      elsif parts != ''
        @f.print '<dt id="', ID_CHAR_EXA, id, '">', hwd, "</dt>\n"
        @f.print "<dd>\n"
        @f.print '<p><i>', parts, "</i></p>\n"
      else
        @f.print '<dt id="', ID_CHAR_EXA, id, '">', hwd, "</dt>\n"
        @f.print "<dd>\n"
        @f.print "<p></p>\n"
      end
    end

    printBody(items)

    @f.print "</dd>\n"
  end

  def printBody(items)

    # ᕶ
    items.each do |item|
      case item['title']
      when 'Extra dictionary examples'
        bullet = EB_DIC_BULLET
      when 'Sentences from books, newspapers, etc.'
        bullet = EB_BOOK_BULLET
      else
        STDERR.print item['title'], "\n"
        bullet = EB_UNKNOWN_BULLET
      end

      item['ex'].each do |ex|
        pos = ex['pos'].to_i
        text = entity2char(ex['text'])
        s, g = gaiziarray(text)
        before = char2entity(s[0...pos])
        after = char2entity(s[pos, s.size])
        s = before + '<b>' + after.sub(/^\S+/){ $& + '</b>' }
        s = s.gsub(/&amp;038;/, '&amp;')  # errata
        if g.size > 0    # O
          @f.print '<p>', bullet, s.gsub(/\x00/){ g.shift }, "</p>\n"
        else
          @f.print '<p>', bullet, s, "</p>\n"
        end
      end
    end

  end

  def entity2char(str)
    str.gsub!(/&quot;/, '"')
    str.gsub!(/&lt;/, '<')
    str.gsub!(/&gt;/, '>')

    str.gsub!(/&amp;/, '&')
    str
  end

  def char2entity(str)
    str.gsub!(/&/, '&amp;')

    str.gsub!(/"/, '&quot;')
    str.gsub!(/</, '&lt;')
    str.gsub!(/>/, '&gt;')
    str
  end

end

### oᕶf[^
class ExampleDic < Example

  def printHTML(id, hwd, items)

    @f.print "<blockquote>\n"
    printBody(items)
    @f.print "</blockquote>\n"

  end

end

### Phrase Bank ᕶ f[^
class ExamplePhrase < Example

  def initialize(outf)
    @f = outf
  end

  def printHTML(id, hwd, items)

    # o
    if hwd =~ /^\n/   # Phrase Bank ᕶ
      hwd = hwd.sub(/\n/, '')
    end
    @f.print '<h2 id="', ID_CHAR_EXA, id, '">&nbsp;&nbsp;', hwd, "</h2>\n"

    @f.print "<blockquote>\n"
    printBody(items)
    @f.print "</blockquote>\n"

  end

end

### Collocation pᕶ f[^
class ExampleColloc < Example

  def initialize(outf)
    @f = outf
  end

  def printHTML(id, hwd, items)

    printBody(items)

  end

end

if __FILE__ == $0
  Home = 'H:/ldoce4/ldoce-data/'

  STDERR.print "Jn: ", Time::now.strftime('%Y-%m-%d %X'), "\n"

  maindoc = MainDoc::new(Home)
  exaf = maindoc.package(PID_EXAMPLE)
  eb = Example::new(STDOUT, maindoc)

  puts '<html>'
  puts '<head>'
  puts '<title>LDOCE4 Examples Bank</title>'
  puts '</head>'
  puts '<body>'
  puts '<dl>'

  n = 0
  exaf.each do |rec|
    STDERR.print n, "\r" if n % 10 == 0
    break if n >= 100
    n += 1
#if n >= 100000
    eb.do(rec.chomp(REC_SEP))
  end

  STDERR.print n, "\n"

  puts '</dl>'
  puts '</body>'
  puts '</html>'

  maindoc.close
##  exaf.close  # sv

  STDERR.print "I: ", Time::now.strftime('%Y-%m-%d %X'), "\n"

end
