if __FILE__ == $0
  scrPath = File::dirname($0.gsub(/\\/, '/')).sub(/\/ldoce4$/, '')
  $LOAD_PATH.unshift(scrPath)     # XNvgpX[hpXɒǉ
end

require 'ldoce4/gaizi'
require 'ldoce4/cfg'
require 'ldoce4/example'

### Dictionary phrases: f[^
class DicPhrase

  SEP_LINE = '' * 10  # ؂
  TITLE = 'Dictionary phrases:'

  def initialize(outf)
    @f = outf
  end

  def close
  end

  def do(source)
    phrs = parse(source)
    printHTML(phrs) if phrs.size > 0
    return phrs
  end

  def parse(source)
    phrs = []

    if source !~ /<div id="divDicphr">/
      STDERR.print 'div id error', "\n"
      return []
    end

    source.scan(%r!<p class="dicphr">(.+?)</p>!m) do |phr, |
      id = title = nil
      if phr =~ /<xblexa id="(\d+)">/
        id = $1
      end
      if phr =~ %r!<img class="puce" .+/>(.+)$!m
        title = $1
      end
      phrs.push({ 'title' => title, 'id' => id })
    end

    return phrs
  end

  def printHTML(phrs)

    @f.print '<p>', SEP_LINE, "</p>\n"
    @f.print '<p><b>', TITLE, "</b></p>\n"
    @f.print "<blockquote>\n"

    phrs.each do |phr|
      targetID = ID_CHAR_EXA + phr['id']
      @f.print '<p><b><a href="', targetID, '">', gaizi(phr['title']), "</a></b></p>\n"
    end

    @f.print "</blockquote>\n"

  end

end

### Phrases from other entries: f[^
class OtherPhrase

  SEP_LINE = '' * 10  # ؂
  TITLE = 'Phrases from other entries:'

  def initialize(outf)
    @f = outf
  end

  def close
  end

  def do(source)
    phrs = parse(source)
    printHTML(phrs) if phrs.size > 0
  end

  def parse(source)
    phrs = []

    if source !~ /<div id="divOthersphr">/
      STDERR.print 'div id error', "\n"
      return []
    end

    source.scan(%r!<p class="dicphrsa">(.+?)</p>!m) do |phr, |
      refid = exaid = hwd = refhwd = nil
      if phr =~ /<xblslso id="(\d+)">/
        refid = $1
      end
      if phr =~ /<xblexa id="(\d+)">/
        exaid = $1
      end
      if phr =~ %r!<img class="puce" .+/>(.+)<span class="dicsee">- see </span> <span class="dicseeAlso">(.+)</span>!m
        hwd = $1; refhwd = $2
      end
      phrs.push('refid' => refid, 'exaid' => exaid, 'hwd' => hwd, 'refhwd' => refhwd)
    end

    return phrs
  end

  def printHTML(phrs)

    @f.print '<p>', SEP_LINE, "</p>\n"
    @f.print '<p><b>', TITLE, "</b></p>\n"
    @f.print "<blockquote>\n"

    phrs.each do |phr|
      hwd = gaizi(phr['hwd'])
      refhwd = gaizi(phr['refhwd'].sub(/_/, ', '))  # i؂
      if phr['exaid'] != nil
        targetID = ID_CHAR_EXA + phr['exaid']
        @f.print '<p><a href="', targetID, '"><b>', hwd, '</b></a> <b>', refhwd, "</b></p>\n"
      else
        @f.print '<p><b>', hwd, '</b> <b>', refhwd, "</b></p>\n"
      end
    end

    @f.print "</blockquote>\n"

  end

end

### Words used with: f[^
class CollocMenu

  SEP_LINE = '' * 10  # ؂
  BULLET = ' '        # ᕶop

  def initialize(outf, maindoc)
    @f = outf
    @maindoc = maindoc

    @ex = ExampleColloc.new(@f)
  end

  def close
  end

  def do(source, mainid)
    items = parse(source)
    printHTML(mainid, items) if items.size > 0
  end

  def parse(source)
    items = []

    if source !~ /<div id="divWords">/
      STDERR.print 'div id error', "\n"
      return []
    end

    divs = source.split('</div>')
    divs.each do |div|
      ps = div.split('</p>')
      ps.each do |p|
        if p =~ %r!<span class="catcorpus">(.+?)</span>!m
          title = $1
          words = []
          items.push({ 'title' => title, 'words' => words })
        end
        if p =~ %r!<xblexa id="(\d+)">.+</xblexa>(.+)\Z!m
          id = $1.to_i
          word = $2
          items.last['words'].push({ 'id' => id, 'word' => word })
        end
      end
    end

=begin
    source.scan(%r!<div .+?>(.+?)</div>!m) do |div, |
      if div =~ %r!<span class="catcorpus">(.+?)</span>!m
        title = $1; phrs = $'
        words = []
        items.push({ 'title' => title, 'words' => words })
        phrs.scan(%r!<p class="corpphr" .+?><xblexa id="(\d+)">.+?</xblexa>(.+?)</p>!m) do |id, word|
          id = id.to_i
          items.last['words'].push({ 'id' => id, 'word' => word })
        end
      end
    end
=end

    return items
  end

  def printHTML(mainid, items)

    # LȖڎf[^邩mF
    contents = ''
    items.each_with_index do |item, idx|
       words = item['words']
       if words.size != 0
         targetID = ID_CHAR + mainid.to_s + '-' + idx.to_s
         selfID = targetID + ID_SUFFIX
         contents += '<p id="' + selfID + '"><b><a href="' + targetID + '">' + gaizi(item['title']) + "</a></b></p>\n"
         contents += '<blockquote><b>'
         words.each do |word|
           contents += gaizi(word['word'])
         end
         contents += "</b></blockquote>\n"
       end
    end
    return if contents == ''

    # o
    title = @maindoc.title(mainid)
    if title =~ /  (\d, )?/      # oAԌԍAiɕ
      hwd = $`; num = $1; parts = $'
    else
      hwd = title; num = nil; parts = ''
    end
    if num != nil
      @f.print '<dt id="', ID_CHAR, mainid.to_s, '" title="', title, '">', hwd, "</dt>\n"
      @f.print "<dd>\n"
      @f.print '<p>', num, '<i>', parts, "</i></p>\n"
    elsif parts != ''
      @f.print '<dt id="', ID_CHAR, mainid.to_s, '">', hwd, "</dt>\n"
      @f.print "<dd>\n"
      @f.print '<p><i>', parts, "</i></p>\n"
    else
      @f.print '<dt id="', ID_CHAR, mainid.to_s, '">', hwd, "</dt>\n"
      @f.print "<dd>\n"
      @f.print "<p></p>\n"
    end

    # ڎ
    @f.print contents
    @f.print '<p>', SEP_LINE, "</p>\n"

    # ᕶ
    items.each_with_index do |item, idx|
       words = item['words']
       if words.size != 0
         selfID = ID_CHAR + mainid.to_s + '-' + idx.to_s
         targetID = selfID + ID_SUFFIX
         @f.print '<p id="', selfID, '"><b><a href="', targetID, '">', gaizi(item['title']), "</a></b></p>\n"
         @f.print '<blockquote>'
         words.each do |word|
           hwd = gaizi(word['word'].sub(/, $/, ''))
           @f.print '<p><b>', BULLET, hwd, "</b></p>\n"
           @f.print '<key type="">', hwd, "</key>\n"
           rec = @maindoc.getrec(word['id'])
           idx = @maindoc.getbodyindex(rec)
           idx.each do |x|
             if x['packid'] == PID_EXAMPLE
               @ex.do(@maindoc.getbody(x).chomp(REC_SEP))
             end
           end
         end
         @f.print "</blockquote>\n"
       end
    end

    @f.print "</dd>\n"
  end

end

###

if __FILE__ == $0

  Home = 'H:/ldoce4/ldoce-data/'
  COLLOC_RECS = 21431

  rs = "\x01"
  n = 0

  maindoc = MainDoc::new(Home)
  phrase = maindoc.package(PID_PHRASE)
  colloc = CollocMenu2::new(STDOUT, maindoc)

  start = Time::now
  STDERR.print "Jn: ", start.strftime('%Y-%m-%d %X'), "\n"

      phrase.each_with_index do |rec, n|

        STDERR.print n, '/', COLLOC_RECS, "\r" if n % 10 == 0
        break if n >= 1000
        n += 1

        data = rec.chomp(REC_SEP)
        if data =~ /<div id="divWords">/
          div = $& + $'
          if data =~ /<div id="(\d+)"/
            id = $1.to_i
            data = div.sub(/<\/div>\z/, '')
            colloc.do(data, id)
          end
        end

      end

    STDERR.print n, '/', COLLOC_RECS, "\n"

  finish = Time::now
  STDERR.print "I: ", finish.strftime('%Y-%m-%d %X'), "\n"
  STDERR.print ": ", Time::at(finish.to_i - start.to_i).utc.strftime('%X'), "\n"

  exit

end
__END__
